import { useEffect, useState, useRef } from "react";
import { Table } from "react-bootstrap";
import { useTranslation } from "react-i18next";
import { faEdit, faTrashAlt } from "@fortawesome/free-regular-svg-icons";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import StaffModal from "../../components/StaffModal";
import { Form } from "react-bootstrap";
import { useForm } from "react-hook-form";
import { CheckIn, Sport } from "../../types";
import { getAllCheckIns, markComment } from "../../api/checkIn";
import { faArrowDown, faArrowUp, faCheck} from "@fortawesome/free-solid-svg-icons";

function Comments() {
  let mounted = useRef(true);
  const { t } = useTranslation();
  const [editElement, setEditElement] = useState<CheckIn | undefined>(undefined);
  const [deleteElement, setDeleteElement] = useState<number>(-1);
  const [checkIns, setcheckIns] = useState<CheckIn[]>([]);
  const [loading, setLoading] = useState<boolean>(false);
  const [commentSearch, setCommentSearch] = useState<string>("");
  //TODO: Change to date?
  const [dateStartSearch, setDateStartSearch] = useState<string>("");
  const [dateEndSearch, setDateEndSearch] = useState<string>("");

  //Default is Date Desc (it only makes sense sorting by date)
  const [sortByDate, setSortByDate] = useState<boolean>(true);
  const [sortByDateAsc, setSortByDateAsc] = useState<boolean>(false);

  const {
    control,
    handleSubmit,
    setValue,
    formState: { errors, isValid },
  } = useForm<Sport>({
    mode: "onBlur",
    reValidateMode: 'onChange'
  });

  async function getcheckIns() {
    const checkIns = await getAllCheckIns();
    if (mounted) {
      setcheckIns(checkIns);
    }
  }

  useEffect(() => {
    getcheckIns();
    return () => {
      mounted.current = false;
    }
  }, []);

  return (
    <div className="m-1 h-100">
      <Table responsive striped bordered hover>
        <thead>
          <tr>
            <th>
              {t("comments.Comment")}
            </th>
            <th
            onClick={() => {
              setSortByDate(true);
			  setSortByDateAsc(!sortByDateAsc);
            }}>
              {t("comments.Date")} {sortByDate && <FontAwesomeIcon icon={!sortByDateAsc ? faArrowDown : faArrowUp} />}
            </th>
            <th></th>
          </tr>
          <tr>
            <th>
            <Form.Control 
                type="text" 
                className="form-control-sm" 
                placeholder="Search" 
                aria-label="Search" 
                onChange={(event) => { 
                  setCommentSearch(event.target.value);
                }}
              />
            </th>
            <th>
            <div className="input-group input-group-sm">
              <div className="me-5">
            <Form.Label>{t("comments.From")}</Form.Label>
            <Form.Control 
                type="date" 
                className="form-control-sm" 
                placeholder="Search" 
                aria-label="Search" 
                onChange={(event) => {
                  setDateStartSearch(event.target.value);
                  if(dateEndSearch !== "" && Date.parse(event.target.value) > Date.parse(dateEndSearch)){
                    setDateStartSearch(dateEndSearch);
                    event.target.value = dateEndSearch;
                  }
                }}
              />
              </div>
              <div>
              <Form.Label>{t("comments.To")}</Form.Label>
              <Form.Control
                type="date"
                className="form-control-sm"
                onChange={(event) => {
                  setDateEndSearch(event.target.value);
                  if(dateStartSearch === "") {
                    setDateEndSearch("");
                    event.target.value = "";
                  }
                  else if(Date.parse(event.target.value) < Date.parse(dateStartSearch)) {
                    setDateEndSearch("");
                    event.target.value = "";
                  }
                }}
              />
              </div>
              </div>
            </th>
          </tr>
        </thead>
        <tbody>
          { /*TODO: Add option for showing done Comments*/
          checkIns
		  .filter((checkIn) => checkIn.returned && checkIn.note && !checkIn.noteDone)
		  .filter((checkIn) => checkIn.note.toLocaleLowerCase().includes(commentSearch.toLocaleLowerCase()))
      .filter((checkIn) => {
        console.log([dateStartSearch, dateEndSearch]);
        if(dateEndSearch !== "" && dateStartSearch !== "") {
          console.log("Test");
          return Date.parse(checkIn.date) >= Date.parse(dateStartSearch) && Date.parse(checkIn.date) <= Date.parse(dateEndSearch);
        }
        else if(dateStartSearch !== "" && dateEndSearch === "") {
          return Date.parse(checkIn.date) === Date.parse(dateStartSearch);
        }
        //dateStartSearch should never be Empty when dateEndSearch is not empty
        else {
          return true;
        }
      })
		  .sort((a, b) => {
			  if(sortByDate && sortByDateAsc) {
				  return Date.parse(b.date) - Date.parse(a.date); 
			  }
			  else {
				  return Date.parse(a.date) - Date.parse(b.date);
			  }
		  })
          .map((x, i) => (
            <tr key={x.id}>
              <td>{x.note}</td>
              <td>
                {new Date(x.date).toLocaleDateString()}
              </td>
              <td>
                <div className="d-flex">
                  <div
                    className="mx-2"
                    onClick={() => {
                      setEditElement(x);
                      setValue('id', x.id);
                      //setValue('name', x.name);
                      //setValue('color', x.color);
                    }}
                  >
                    <FontAwesomeIcon
                      icon={faEdit}
                      className="text-secondary clickableIcon"
                      size="1x"
                    />
                  </div>
                  <div
                    className="mx-2"
                    onClick={() => {
                      markComment(true, x.id);
                      //setDeleteElement(checkIns.indexOf(x));
                    }}
                  >
                    <FontAwesomeIcon
                      icon={faCheck}
                      className="text-success clickableIcon"
                      size="1x"
                      title={t("comments.MarkDone")}
                    />
                  </div>
                </div>
              </td>
            </tr>
          ))}
        </tbody>
      </Table>
      {/*TODO: Is this needed? 
      <StaffModal
        header={t("comments.EditComment")}
        show={!!editElement}
        successText={t("common.Edit")}
        loadingNext={loading}
        disableNext={!isValid}
        onHide={() => {
          setEditElement(undefined);
        }}
        onSuccess={() => {
          setLoading(true);
          handleSubmit((data) => {
            editComment(data).then(async () => {
                await getcheckIns();
                setEditElement(undefined);
                setLoading(false);
              });
          })()
        }}
      >
        <Form>
          <Controller
            name="id"
            control={control}
            defaultValue={editElement?.id}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("sports.Id")}</Form.Label>
                <Form.Control disabled type="text" {...field} />
              </div>
            )}
          />
          <Controller
            name="name"
            control={control}
            defaultValue={editElement?.note}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("sports.Name")}</Form.Label>
                <Form.Control type="text" {...field} isInvalid={!!errors.name} />
                <Form.Control.Feedback type="invalid">
                  {errors.name?.message}
                </Form.Control.Feedback>
              </div>
            )}
            rules={{
              required: {
                value: true,
                message: t("common.messages.required", {
                  val: t("Sports.Name"),
                }),
              },
            }}
          />
        </Form>

		</StaffModal> */}
      <StaffModal
        header={t("comments.DeleteComment")}
        hideColor="secondary"
        successText={t("common.Delete")}
        successColor="danger"
        show={deleteElement !== -1}
        loadingNext={loading}
        onHide={() => {
          setDeleteElement(-1);
        }}
        onSuccess={async () => {
          setLoading(true);
		  //TODO: 
          //await deleteSportInApi({ id: sports[deleteElement].id })
          //await getSports();
          setDeleteElement(-1);
          setLoading(false);
        }}
      >
        <span>
          {t("comments.messages.DeleteText", { val: checkIns[deleteElement]?.date /*sports[deleteElement]?.name*/ })}
        </span>
      </StaffModal>
    </div>
  );
}

export default Comments;
